package Jcg.convexhull2d;

import Jcg.geometry.*;
import Jcg.geometry.kernel.*;

import java.util.*;

public class AndrewConvexHull implements ConvexHull_2 {
	
	class SortPointsByCoordinates implements Comparator<Point_2> {
		
		public int compare(Point_2 p1, Point_2 p2) {
			return p1.compareTo(p2);
		}
	}
	
    public ArrayList<Point_2> computeUpperHull(ArrayList<Point_2> sortedPoints) {
    	ArrayList<Point_2> upperHull = new ArrayList<Point_2>();
    	for (int i=0; i<sortedPoints.size(); i++) {
    		Point_2 p=sortedPoints.get(i);
    		int last=upperHull.size();
    		GeometricPredicates_2 predicates=new FilteredPredicates_2();
    		while(upperHull.size()>=2 && 
			  	  //!GeometricOperations_2.isCounterClockwise(upperHull.get(last-1),upperHull.get(last-2),p)
    				!predicates.isCounterClockwise(upperHull.get(last-1),upperHull.get(last-2),p)
			  	  ) {
				//vectorProduct(upperHull.get(last-1),upperHull.get(last-2),p)<=0) {
				upperHull.remove(last-1); // remove last point in the upper hull
				last=upperHull.size();
			}
			upperHull.add(upperHull.size(),p); // add at the end
    	}
		return upperHull;
    }

    public ArrayList<Point_2> computeLowerHull(ArrayList<Point_2> sortedPoints) {
    	ArrayList<Point_2> lowerHull = new ArrayList<Point_2>();
    	for (int i=0; i<sortedPoints.size(); i++) {
    		Point_2 p=sortedPoints.get(i);
    		int last=lowerHull.size();
    		GeometricPredicates_2 predicates=new FilteredPredicates_2();
    		while(lowerHull.size()>=2 && 
			  	  //GeometricOperations_2.isCounterClockwise(lowerHull.get(last-1),lowerHull.get(last-2),p)
			  	  predicates.isCounterClockwise(lowerHull.get(last-1),lowerHull.get(last-2),p)
			     ) {
				//vectorProduct(upperHull.get(last-1),upperHull.get(last-2),p)<=0) {
				lowerHull.remove(last-1); // remove last point in the upper hull
				last=lowerHull.size();
			}
			lowerHull.add(lowerHull.size(),p); // add at the end
    	}
		return lowerHull;
    }
        
    public PointCloud_2 computeConvexHull(PointCloud_2 points) {
    	System.out.print("Jcg - computing 2D convex hull (Andrew algorithm)... ");
    	ArrayList<Point_2> sortedPoints=(ArrayList<Point_2>)points.listOfPoints();	
    	Collections.sort(sortedPoints,new SortPointsByCoordinates());
    	
    	ArrayList<Point_2> upperHull=computeUpperHull(sortedPoints);
    	ArrayList<Point_2> lowerHull=computeLowerHull(sortedPoints);
    	
    	for(int i=lowerHull.size()-1;i>=0;i--) {
    		upperHull.add(lowerHull.get(i));
    	}
    	System.out.println("done");
    	return new PointCloud_2(upperHull);
    }

}