package Jcg.convexhull3d;

import java.util.ArrayList;
import java.util.Collection;

import Jcg.geometry.PointCloud_3;
import Jcg.geometry.Point_3;
import Jcg.geometry.Triangle_3;
import Jcg.geometry.kernel.GeometricPredicates_3;
import Jcg.geometry.kernel.FilteredPredicates_3;

/**
 * Slow computation of 3D convex hull (trivial exhaustive method)
 *
 * @author Luca Castelli Aleardi, Ecole Polytechnique (INF562)
 * @version feb 2014
 */
public class SlowConvexHull_3 {

	/**
	 * Compute the 3D convex hull by checking all triangles
	 * Warning: to be completed
	 * 
	 * @param points the input 3D point cloud (stored as an array list)
	 * @return the collection of triangles on the boundary of the 3D convex hull
	 */
    public Collection<Triangle_3> computeConvexHull(PointCloud_3 points) {
    	int n=points.size();
		Collection<Triangle_3> result=new ArrayList<Triangle_3>(); // the result
		Point_3[] array=new Point_3[n]; // store points into an array
		
		for(int i=0;i<n;i++) { // iterate over all possible triangles
			for(int j=i+1;j<n;j++) {
				for(int k=j+1;k<n;k++) {
					Triangle_3 t=new Triangle_3(array[i], array[j], array[k]);
				}
			}
		}
		return result;
    }
    
	/**
	 * Check whether all points lie in the same half-space with respect to a given triangle
	 * 
	 * @param t a given triangle
	 * @param points the input 3D point cloud (stored as an array list)
	 * @return true whether all points lie in the same half-space defined by the supporting hyperplane of t
	 */
    private boolean isOnBoundary(Triangle_3 t, Point_3[] points) {
    	if(points==null)
    		throw new Error("Error: point cloud not defined");
    	if(points.length==0)
    		return true;
    	
    	GeometricPredicates_3 predicates=new FilteredPredicates_3();
    	int orientation=predicates.orientation(t.vertex(0), t.vertex(1), t.vertex(2), points[0]);
    	for(int i=1;i<points.length;i++) {
    		int newOrientation=predicates.orientation(t.vertex(0), t.vertex(1), t.vertex(2), points[i]);
    		if(newOrientation==0) // coplanar point
    			continue;
    		else if(orientation<0 && newOrientation<0) // on the same (negative side)
    			continue;
    		else if(orientation>0 && newOrientation>0) // on the same (positive side)
    			continue;
    		else if(orientation<0 && newOrientation>0) // on different sides
    			return false;
    		else if(orientation>0 && newOrientation<0) // on different sides
    			return false;
    		else
    			orientation=newOrientation;
    	}
    	return true;
    }


}
