package Jcg.graph;

import java.util.*;

/**
 * Pointer based implementation of an Adjacency List Representation of a graph
 * 
 * @author Luca Castelli Aleardi
 *
 */
public class AdjacencyListGraph<V extends Node> implements Graph<V> {
	
	public ArrayList<V> vertices;
	
	public AdjacencyListGraph() {
		this.vertices=new ArrayList<V>();
	}

	public AdjacencyListGraph(int d) {
		this.vertices=new ArrayList<V>(d);
	}
	
	public void addNode(V v) {
		this.vertices.add(v);
	}

	public void removeNode(V v) {
		if(this.vertices.contains(v)==false)
			return;
		for(V u: this.getNeighbors(v)) { // remove all edges between v and its neighbors
			u.removeNeighbor(v);
		}
		this.vertices.remove(v); // remove the vertex from the graph
	}

    public void addEdge(V a, V b) {
    	if(a==null || b==null)
    		return;
    	a.addNeighbor(b);
    	b.addNeighbor(a);
    }

    public void removeEdge(V a, V b){
    	if(a==null || b==null)
    		return;
    	a.removeNeighbor(b);
    	b.removeNeighbor(a);
    }
    
    public boolean adjacent(V a, V b) {
    	if(a==null || b==null)
    		throw new Error("Graph error: vertices not defined");
    	return a.adjacent(b);
    }
    
    public int degree(V v) {
    	return v.degree();
    }
    
    public Collection<V> getNeighbors(V v) {
    	return v.neighborsList();
    }
        
    public int sizeVertices() {
    	return this.vertices.size();
    }
	

}
