package Jcg.graph;

import Jcg.geometry.*;

import java.util.Collection;
import java.util.ArrayList;

/**
 * Implementation of a geometric graph (generic and undirected)
 *
 * @author Luca Castelli Aleardi, Ecole Polytechnique (INF562)
 * @version dec 2012
 */
public interface GeometricGraph<X extends Point_> extends Graph<Node<X>> {
	
	/**
     * Set the geometric coordinates of a vertex
     */		
	public void setPoint(Node<X> v, X p);

	/**
     * Return the geometric coordinates of a vertex
     */		
	public X getPoint(Node<X> v);
	
	/**
     * Add a new vertex to the graph (with its geometric coordinates)
     */		
	public void addNode(X p);
	
	/**
     * Return the list of points associated to the vertices of the graph
     */		
	public ArrayList<X> listOfPoints();

	/**
     * Compute the bounding box (enclosing the vertices of the graph)
     * 
     * @return X[] the array containing the min and max bounds
     */		
	public X[] boundingBox();

}
