package Jcg.graph;

import Jcg.geometry.*;

import java.util.*;

/**
 * Implementation of a geometric graph, whose vertices correspond to 2D points
 *
 * @author Luca Castelli Aleardi, Ecole Polytechnique (INF562)
 * @version dec 2012
 */
public class GeometricGraph_2 extends AdjacencyListGraph<Node<Point_2>> implements GeometricGraph<Point_2> {

	/**
     * Create a new graph
     */		
	public GeometricGraph_2() {
		super();
	}

	/**
     * Create a graph of size n
     * @param n the number of vertices
     */		
	public GeometricGraph_2(int n) {
		super(n);
	}
	
	/**
     * Set the geometric coordinates of a vertex
     */		
	public void setPoint(Node<Point_2> v, Point_2 p) {
		v.setData(p);
	}
	
	/**
     * Return the geometric coordinates of a vertex
     * @param v a vertex in the graph
     */		
	public Point_2 getPoint(Node<Point_2> v) {
		return v.getData();
	}
	
	/**
     * Add a new vertex to the graph (with its geometric coordinates)
     * @param p the coordinates of the vertex to add
     */		
	public void addNode(Point_2 p) {
		Node<Point_2> v=new Node<Point_2>();
		setPoint(v, p);
		this.addNode(v);
	}

	/**
     * compute and return the bounding box for the points set.
     * The result is stored in a table.
     */		
    public Point_2[] boundingBox() {
    	PointCloud_2 points=new PointCloud_2(this.listOfPoints());
    	return points.boundingBox();
    }

    /**
     * Return the list of edges of the graph (their corresponding segments)
     */		   
    public List<Point_2[]> computeEdges() {
    	List<Point_2[]> result=new ArrayList<Point_2[]>();
    	Point_2[] segment;
    	for(Node<Point_2> u: this.vertices) {
    		for(Node<Point_2> v: u.neighbors) {
    			segment=this.getSegment(u, v);
    			result.add(segment);
    		}
    	}
    	return result;
    }

    /**
     * Return the list of vertex points of the graph
     */		   
    public ArrayList<Point_2> listOfPoints() {
    	ArrayList<Point_2> result=new ArrayList<Point_2>();
    	for(Node<Point_2> u: this.vertices) {
    		if(u!=null && this.getPoint(u)!=null)
    			result.add(this.getPoint(u));
    	}
    	return result;
    }

    /**
     * Return the straight-line segment having as extremities two given vertices of the graph
     * @param u first vertex of the edge
     * @param v the other vertex of the edge
     */		   
    public Point_2[] getSegment(Node<Point_2> u, Node<Point_2> v) {
    	Point_2[] result=new Point_2[2];
    	result[0]=u.getData();
    	result[1]=v.getData();
    	return result;
    }
    
}
