package Jcg.graph;

import java.util.*;

import Jcg.geometry.*;

/**
 * Implementation of a geometric graph, whose vertices correspond to 3D points
 *
 * @author Luca Castelli Aleardi, Ecole Polytechnique (INF562)
 * @version dec 2012 
 */
public class GeometricGraph_3 extends AdjacencyListGraph<Node<Point_3>> implements GeometricGraph<Point_3>{

	public GeometricGraph_3() {
		super();
	}

	public GeometricGraph_3(int n) {
		super(n);
	}

	public void setPoint(Node<Point_3> v, Point_3 p) {
		v.setData(p);
	}
	
	public Point_3 getPoint(Node<Point_3> v) {
		return v.getData();
	}

	/**
     * Add a new vertex to the graph (with its geometric coordinates)
     */		
	public void addNode(Point_3 p) {
		Node<Point_3> v=new Node<Point_3>();
		setPoint(v, p);
		this.addNode(v);
	}

	/**
     * Compute and return the bounding box for the points set.
     */		
    public Point_3[] boundingBox() {
    	throw new Error("To be completed");
    }

    /**
     * Return the list of edges of the graph (their corresponding segments)
     */		   
    public List<Point_3[]> computeEdges() {
    	List<Point_3[]> result=new ArrayList<Point_3[]>();
    	Point_3[] segment;
    	for(Node<Point_3> u: this.vertices) {
    		for(Node<Point_3> v: u.neighbors) {
    			segment=this.getSegment(u, v);
    			result.add(segment);
    		}
    	}
    	return result;
    }
    
    /**
     * Return the list of vertex points of the graph
     */		   
    public ArrayList<Point_3> listOfPoints() {
    	ArrayList<Point_3> result=new ArrayList<Point_3>();
    	for(Node<Point_3> u: this.vertices) {
    		if(u!=null && this.getPoint(u)!=null)
    			result.add(this.getPoint(u));
    	}
    	return result;
    }

    /**
     * Return the segment having as extremities two given vertices of the graph
     */		   
    public Point_3[] getSegment(Node<Point_3> u, Node<Point_3> v) {
    	Point_3[] result=new Point_3[2];
    	result[0]=u.getData();
    	result[1]=v.getData();
    	return result;
    }

}
