/**
 * 
 */
package Jcg.graph;

import java.util.HashMap;
import java.util.List;

import Jcg.polyhedron.*;
import Jcg.geometry.*;
import Jcg.mesh.*;
import Jcg.triangulations3D.*;

/**
 * @author Luca Castelli Aleardi
 *
 */
public class GraphBuilder {

    /**
     * Construct the graph corresponding to a 2D triangulation.
     * It requires the list of points corresponding
     * to the vertices of the graph.
     */		   
    public static GeometricGraph_2 getGraphFrom2DTriangulation(List<Point_2> vertices, List<Point_2[]> faces) {
    	System.out.print("constructing geometric graph from 2D triangulation: ");
    	System.out.print(""+vertices.size()+" vertices ...");
    	// store vertex indices in map 
    	HashMap <Point_2,Integer> vert = new HashMap<Point_2,Integer> ();
    	
    	GeometricGraph_2 graph=new GeometricGraph_2();

    	 int i=0;
    	 for (Point_2 p : vertices) {
    		 graph.addNode(new Node<Point_2>(p));
    	     vert.put(p, new Integer(i++));
    	 }    
    	 
    	 for (Point_2[] f : faces) {
    	     Integer[] ind = new Integer [f.length];
    	     int j=0;
    	     for (; j<f.length; j++) {
    		 ind[j] =  vert.get(f[j]);
    		 if (ind[j] == null)
    		     break;
    	     }
    	     if (j < f.length) {
    		 System.out.println ("Index pb in faces set in writeToFile().");
    		 System.exit (-1);
    	     }
    	     
    	     Node<Point_2> v1,v2 ;
    	     for (j=0; j<f.length-1; j++) {
    	    	 /*v1=graph.getNode(ind[j]);
    	    	 v2=graph.getNode(ind[j+1]);
    	    	 v1.addNeighbors(v2);
    	    	 v2.addNeighbors(v1);*/
    	     }
	    	 /*v1=graph.getNode(ind[f.length-1]);
	    	 v2=graph.getNode(ind[0]);
	    	 v1.addNeighbors(v2);
	    	 v2.addNeighbors(v1);*/
    	 }
    	 System.out.println(" done");
    	 System.out.println("vertices: "+graph.sizeVertices());
    	 return graph;
    }
    
    /**
     * Construct the graph corresponding to a 3D mesh (OFF file).
     * It requires all faces are triangles.
     */		   
    public static GeometricGraph_3 getGraphFrom3DMesh(String filename) {
    	System.out.println("constructing geometric graph from an OFF file...");
		SharedVertexRepresentation m=new SharedVertexRepresentation(filename);
    	
    	//System.out.println("m.vertices "+m.points.length);
    	//System.out.println("m.faces "+m.faces.length);
    	//System.out.println("m.halfedges "+m.sizeHalfedges);    	
    	
    	GeometricGraph_3 g3D=new GeometricGraph_3();
    	for(int i=0;i<m.sizeVertices;i++) {
    		Point_3 p=m.points[i];
    		/*g3D.addNode(p);*/
    	}
    	
    	for(int i=0;i<m.faces.length;i++) {
    		int index1=m.faces[i][0];
    		int index2=m.faces[i][1];
    		int index3=m.faces[i][2];
    		//System.out.println(""+index1+" "+index2+" "+index3);
    		/*GraphNode<Point_3> v1=g3D.getNode(index1);
    		GraphNode<Point_3> v2=g3D.getNode(index2);
    		GraphNode<Point_3> v3=g3D.getNode(index3);*/
    		/*v1.addNeighbors(v2);
    		v2.addNeighbors(v3);
    		v3.addNeighbors(v1);*/
    		
    	}
    	
    	System.out.println("3D geometric graph construction... done");
    	return g3D;
    }

    /**
     * Construct the graph corresponding to a 3D triangulation.
     */		   
    public static GeometricGraph_3 getGraphFrom3DTriangulation(Delaunay_3 tri) {
    	System.out.print("constructing geometric graph from 3D triangulation: ");
    	
    	GeometricGraph_3 graph=new GeometricGraph_3();
    	return graph;
    }

}
