package Jcg.graph;

import java.util.ArrayList;
import java.util.List;

/**
 * A class for representing a node of a (dynamic) graph, with associated data
 *
 * @author Luca Castelli Aleardi
 */
public class Node<X> {

    /** List of neighbors of a node in the graph */
	public ArrayList<Node<X>> neighbors=null;
    /** the data stored in the node (e.g. geometric coordinates) */
	X data;
	
    /** Tag of a node: useful for storing additional information associated to a node */
	public int tag;
    /** Index of a node: it can be useful to index nodes */
	public int index;
	

	    public Node() { 
	    	this.neighbors=new ArrayList<Node<X>>();
	    }
	    
	    public Node(X data) {
	    	this.data=data;
	    }

	    /**
	     * Add a new node 'v' to the neighbors
	     * 
	     * Remark: loops and multiple edges are not allowed
	     */
	    public void addNeighbor(Node<X> v) {
	    	if(v!=this && this.neighbors.contains(v)==false)
	    		this.neighbors.add(v);
	    }

	    public void removeNeighbor(Node<X> v) {
	    	this.neighbors.remove(v);
	    }

	    public boolean adjacent(Node<X> v) {
	    	return this.neighbors.contains(v);
	    }
	    
	    public List<Node<X>> neighborsList() {
	    	return this.neighbors;
	    }
	    
	    /**
	     * Set the tag of a vertex
	     */
	    public void setTag(int tag) { 
	    	this.tag=tag;
	    }  
	    
	    /**
	     * Return the tag of a vertex
	     */
	    public int getTag() { 
	    	return this.tag; 
	    }
	    
	    public void setData(X data) {
	    	this.data=data;
	    }
	    
	    public X getData() {
	    	return this.data;
	    }

	    /**
	     * Return the degree of a node
	     */
	    public int degree() {
	    	return this.neighbors.size();
	    }
	    
	    public String toString(){
	        return "vert. "+tag;
	    }

}
