package Jcg.simplification;

import java.util.Comparator;
import java.util.HashMap;

import Jama.Matrix;
import Jcg.geometry.Point_3;
import Jcg.polyhedron.Halfedge;
import Jcg.polyhedron.Vertex;

class HalfEdgeComparator implements Comparator<Halfedge<Point_3>>{

	@Override
	public int compare(Halfedge<Point_3> e1, Halfedge<Point_3> e2) {
		
		double l1,l2;
		Point_3 p1A, p1B, p2A, p2B;
		p1A = e1.getVertex().getPoint();
		p1B = e1.opposite.getVertex().getPoint();
		p2A = e2.getVertex().getPoint();
		p2B = e2.opposite.getVertex().getPoint();
		
		l1 = Math.pow((p1A.x - p1B.x), 2) +  Math.pow((p1A.y - p1B.y), 2) + Math.pow((p1A.z - p1B.z), 2);
		l2 = Math.pow((p2A.x - p2B.x), 2) +  Math.pow((p2A.y - p2B.y), 2) + Math.pow((p2A.z - p2B.z), 2);
		
		if(l1>l2) return 1;
		if(l1<l2) return -1;
		return 0;
	}
	
}
	
	
class HalfEdgeComparatorQEMnew implements Comparator<Halfedge<Point_3>>{
	HashMap<Vertex<Point_3>,Matrix> Qmap;
	
	HalfEdgeComparatorQEMnew(HashMap<Vertex<Point_3>,Matrix> Qmap){
		this.Qmap = Qmap;
	}
	
	@Override
	public int compare(Halfedge<Point_3> e1, Halfedge<Point_3> e2) {
	
		Vertex<Point_3> ver1A, ver1B, ver2A, ver2B;
		Point_3 p1A, p1B, p2A, p2B;
		
		
		ver1A = e1.getVertex();
		ver1B = e1.opposite.getVertex();
		ver2A = e2.getVertex();
		ver2B = e2.opposite.getVertex();
		
		p1A = ver1A.getPoint();
		p1B = ver1B.getPoint();
		p2A = ver2A.getPoint();
		p2B = ver2B.getPoint();
		
		double[][] v1A = {{p1A.x}, {p1A.y}, {p1A.z}, {1}};
		double[][] v1B = {{p1B.x}, {p1B.y}, {p1B.z}, {1}};
		double[][] v2A = {{p2A.x}, {p2A.y}, {p2A.z}, {1}};
		double[][] v2B = {{p2B.x}, {p2B.y}, {p2B.z}, {1}};
		
		Matrix V1A = new Matrix(v1A);
		Matrix V1B = new Matrix(v1B);
		Matrix V2A = new Matrix(v2A);
		Matrix V2B = new Matrix(v2B);
		
		Matrix Qfinal1 = Qmap.get(ver1A).plus(Qmap.get(ver1B));
		Matrix Qfinal2 = Qmap.get(ver2A).plus(Qmap.get(ver2B));
		
		Matrix Vbar1, Vbar2;
		
		Matrix Qsolution1 = Qfinal1.copy();
		Matrix Qsolution2 = Qfinal2.copy();
		
		Qsolution1.set(3, 0, 0);
		Qsolution1.set(3, 1, 0);
		Qsolution1.set(3, 2, 0);
		Qsolution1.set(3, 3, 1);
		
		Qsolution2.set(3, 0, 0);
		Qsolution2.set(3, 1, 0);
		Qsolution2.set(3, 2, 0);
		Qsolution2.set(3, 3, 1);
		
		Matrix Bvector = new Matrix(new double[][] {{0},{0},{0},{1}});
		
		double precision = 0.000001f;
		if(Math.abs(Qsolution1.det())<precision) Vbar1 = V1A.plus(V1B).times(0.5f);
		else Vbar1 = Qsolution1.inverse().times(Bvector);
		if(Math.abs(Qsolution2.det())<precision) Vbar2 = V2A.plus(V2B).times(0.5f);
		else Vbar2 = Qsolution2.inverse().times(Bvector);
		
		double error1 = Vbar1.transpose().times(Qfinal1).times(Vbar1).get(0, 0);
		double error2 = Vbar2.transpose().times(Qfinal2).times(Vbar2).get(0, 0);

		
		if(error1>error2) return 1;
		if(error1<error2) return -1;
		return 0;
	}

	
	
}
