package Jcg.test;
import tc.TC;
import Jcg.mesh.MeshLoader;
import Jcg.polyhedron.Halfedge;
import Jcg.polyhedron.Polyhedron_3;
import Jcg.schnyderwoods.PlanarTriSchnyderWood;

public class ComputeSchnyderDecomposition {

    /**
     * Compute the encoding of a planar triangulation (encowed with its minimal Schnyder wood)
     * 
     * @param filename
     * 			the name of the file storing the triangulation (OFF format)
     * @param rootEdge
     * 			the integer index of the root edge (between 0..2e-1)
     * @param outputFile
     * 			output file storing the encoding
     */
	public static void encode(String filename, int rootEdge, String outputFile) { // choice of the root edge (edge 2, for the triangulation_9_JCB2014.off)
		//System.out.println("\nComputing Schnyder decomposition");
		
		Polyhedron_3 mesh=MeshLoader.getPlanarMesh(filename);
		PlanarTriSchnyderWood cut;
		cut=new PlanarTriSchnyderWood(mesh, ((Halfedge) mesh.halfedges.get(rootEdge)).getNext()); // choice of the root face
		cut.performTraversal(); // computing the Schnyder orientation
		
		String[] encoding=cut.encodeSchnyderWood();
		TC.ecritureDansNouveauFichier(outputFile+".par");
		System.out.println("T_0 encoding ("+encoding[0].length()+" symbols)"); 
		TC.println(encoding[0]);
		System.out.println("T_2 encoding ("+encoding[1].length()+" symbols)"); 
		TC.println(encoding[1]);
		TC.ecritureSortieStandard();
		
		DickPathsFromParenthesisWords(encoding[0], encoding[1], outputFile+".dyck");
		//SchnyderTreeToFile(cut, 0, "tree0.txt");
		//SchnyderTreeToFile(cut, 1, "tree1.txt");
		//SchnyderTreeToFile(cut, 2, "tree2.txt");
		int innerVertices=mesh.sizeOfVertices()-3;
		System.out.println("Triangulation (with "+innerVertices+" inner vertices) encoded into files: ");
		System.out.println("\t"+outputFile+".par");
		System.out.println("\t"+outputFile+".dyck");
	}
	
    /**
     * Compute the two non-crossing Dick paths corresponding to a Schnyder wood
     * 
     * @param T0
     * 			parenthesis encoding of tree T0
     * @param T2
     * 			parenthesis encoding of tree T2
     * @param outputFile
     * 			output file storing the heights of the two Dick paths
     */
	public static void DickPathsFromParenthesisWords(String T0, String T2, String outputFile) { 
		TC.ecritureDansNouveauFichier(outputFile);
		int h0=0;
		int h2=2;
		
		TC.println(T0.length()+"   2"); // file header 
		TC.println(h0+"   "+h2); // initial heights of the two Dick paths
		for(int i=0;i<T0.length();i++) {
			if(T0.charAt(i)=='(')
				h0++;
			else
				h0--;
			if(T2.charAt(i)=='[')
				h2++;
			else
				h2--;
			TC.println(h0+"   "+h2);
		}
			
		TC.ecritureSortieStandard();
	}

	public static void SchnyderTreeToFile(PlanarTriSchnyderWood cut, int i, String outputFile) {
		System.out.print("Saving Schnyder tree (file "+outputFile+") ...");
		Polyhedron_3 mesh=cut.polyhedron; // the input mesh
		TC.ecritureDansNouveauFichier(outputFile);
		
		for(Object o: mesh.halfedges) { // 
			Halfedge e=(Halfedge)o;
			int v0=e.vertex.index;
			int v1=e.getOpposite().getVertex().index;
			int color=cut.edgeColor[e.index];
			if(v0<v1 && color==i)
				TC.println(""+v0+" "+v1);
		}
		
		TC.ecritureSortieStandard();
		System.out.println("done");
	}
	
	public static void main(String[] args) {
		if(args.length<3) {
			System.out.println("Wrong number of input parameters:");
			System.out.println("Usage example: \n\t data/mesh.off 2 encoding.dat");
			return;
		}
		encode(args[0], Integer.parseInt(args[1]), args[2]);
	}
	
}
