package Jcg.test;
import Jcg.meshgeneration.SphereGeneration;
import Jcg.meshgeneration.StackedGeneration;

public class GenerateMesh {

    public static void main (String[] args) {
    	System.out.println("Mesh generation (Luca Castelli Aleardi, 2019)");
    	System.out.println("This program allows to generate a few classes of synthetic and random meshes.");
    	System.out.println("The resulting mesh is provided with a geometric 3D embedding and stored in OFF format.\n");
    	if (args.length<2) {
    		System.out.println("Warning: at least two input parameters required");
    		System.out.println("Input arguments include:");
    		System.out.println("\t -sphere iterations");
    		System.out.println("\t -globe width height");
    		System.out.println("\t -grid width height");
    		System.out.println("\t -cylinder width height");
    		System.out.println("\t -random size");
    		System.out.println("\t -delaunay size");
    		System.out.println("\t -stacked iterations");
    		System.out.println("Usage example 1: java GenerateMesh -sphere 2");
    		System.out.println("Usage example 2: java GenerateMesh -globe 100 100");
    		System.out.println("Usage example 3: java GenerateMesh -random 1000");
    		return;
    	}
    	
    	if(args[0].equals("-sphere")) {
    		int iterations=Integer.parseInt(args[1]);
    		SphereGeneration sphere=new SphereGeneration();
    		sphere.run(iterations);
    		Jcg.mesh.IO.writePolyedronToOFF(sphere.polyhedron3D, "sphere.off");
    	}
    	else if(args[0].equals("-stacked")) {
    		int iterations=Integer.parseInt(args[1]);
    		StackedGeneration stacked=new StackedGeneration();
    		stacked.subdivideAllFaces(iterations);
    		Jcg.mesh.IO.writePolyedronToOFF(stacked.polyhedron3D, "stacked.off");
    	}
        
    }

}
