package Jcg.test;
import Jcg.util.*;

import java.util.BitSet;

public class TestBitSet {
	
	public static void showMemory() {
		Runtime runtime = Runtime.getRuntime();  
		  
		long maxMemory = runtime.maxMemory();  
		long allocatedMemory = runtime.totalMemory();  
		long freeMemory = runtime.freeMemory();  
		  
		System.out.print("free memory: " + freeMemory / 1024+"k");  
		System.out.print(", allocated: " + allocatedMemory / 1024+"k");  
		System.out.print(", max: " + maxMemory /1024+"k");  
		System.out.println(", total free memory: " + (freeMemory + (maxMemory - allocatedMemory)) / 1024+"k");	
	}

	public static void benchJavaBitSet(int n) {
		long time0=System.currentTimeMillis();
		BitSet bits=new BitSet(n);
		int repeat=50;
		
		// writing
		for(int j=0;j<repeat;j++) {
			for(int i=0;i<n;i++) {
				bits.set(i);
			}
		}
		long time1=System.currentTimeMillis();

		// reading
		boolean value;
		for(int j=0;j<repeat;j++) {
			for(int i=0;i<n;i++) {
				value=bits.get(i);
			}
		}

		long time2=System.currentTimeMillis();
		System.out.println("time: "+((time1-time0)/1000.)/repeat);
		System.out.println("time: "+((time2-time1)/1000.)/repeat);
		System.out.println("test done");
	}
	
	public static void benchBitSet32(int n) {
		long time0=System.currentTimeMillis();
		BitSet32 bits=new BitSet32(n);
		int repeat=50;
		
		// writing
		for(int j=0;j<repeat;j++) {
			for(int i=0;i<n;i++) {
				bits.set(i);
			}
		}
		long time1=System.currentTimeMillis();

		// reading
		boolean value;
		for(int j=0;j<repeat;j++) {
			for(int i=0;i<n;i++) {
				value=bits.get(i);
			}
		}

		long time2=System.currentTimeMillis();
		System.out.println("time: "+((time1-time0)/1000.)/repeat);
		System.out.println("time: "+((time2-time1)/1000.)/repeat);
		System.out.println("test done");
	}

	public static void benchBitSet64(int n) {
		long time0=System.currentTimeMillis();
		BitSet64 bits=new BitSet64(n);
		int repeat=50;
		
		// writing
		for(int j=0;j<repeat;j++) {
			for(int i=0;i<n;i++) {
				bits.set(i);
			}
		}
		long time1=System.currentTimeMillis();

		// reading
		boolean value;
		for(int j=0;j<repeat;j++) {
			for(int i=0;i<n;i++) {
				value=bits.get(i);
			}
		}

		long time2=System.currentTimeMillis();
		System.out.println("time: "+((time1-time0)/1000.)/repeat);
		System.out.println("time: "+((time2-time1)/1000.)/repeat);
		System.out.println("test done");
	}

	public static void benchBooleanArray(int n) {
		
		long time0=System.currentTimeMillis();
		boolean[] b=new boolean[n];
		int repeat=50;
		
		// writing
		for(int j=0;j<repeat;j++) {
			for(int i=0;i<n;i++) {
				b[i]=true;
			}
		}
		long time1=System.currentTimeMillis();

		// reading
		boolean value;
		for(int j=0;j<repeat;j++) {
			for(int i=0;i<n;i++) {
				value=b[i];
			}
		}

		long time2=System.currentTimeMillis();
		System.out.println("time: "+((time1-time0)/1000.)/repeat);
		System.out.println("time: "+((time2-time1)/1000.)/repeat);
		System.out.println("test done");
	}
	
	static int logBase2(int n) {
		double a=(double)n;
		return (int)((Math.log10(a))/Math.log10(2.));
	}

	public static void correctnessBitSet32() {
		  System.out.println("Testing BitSet32");		
		  int size=50;
		  BitSet32 bitSet=new BitSet32(size);
		  
		  boolean[] nonZeroBits=new boolean[size];
		  double r;
		  for(int i=0;i<size;i++) {
			  r=Math.random();
			  if(r<0.5)
				  nonZeroBits[i]=true;
		  }
		  System.out.println("Original bit vector:\n"+booleanToString(nonZeroBits));
		  
		  System.out.println("BitSet32:\n"+bitSet);
		  
		  // initialization
		  for(int i=0;i<size;i++) {
			  if(nonZeroBits[i]==true)
				  bitSet.set(i);
		  }
		  System.out.println("BitSet32 initialized:\n"+bitSet);
		  
		  // reading/access
		  boolean bitValue;
		  boolean[] resultingBitVector=new boolean[size];
		  for(int i=0;i<size;i++) {
			  resultingBitVector[i]=bitSet.get(i);
		  }
		  System.out.println("Resulting bit vector\n"+booleanToString(resultingBitVector));
		  
		  System.out.println("test done");
	}

	public static void correctnessBitSet64() {
		  System.out.println("Testing BitSet64");		
		  int size=100;
		  BitSet64 bitSet=new BitSet64(size);
		  
		  boolean[] nonZeroBits=new boolean[size];
		  double r;
		  for(int i=0;i<size;i++) {
			  r=Math.random();
			  if(r<0.5)
				  nonZeroBits[i]=true;
		  }
		  System.out.println("Original bit vector:\n"+booleanToString(nonZeroBits));
		  
		  System.out.println("BitSet64:\n"+bitSet);
		  
		  // initialization
		  for(int i=0;i<size;i++) {
			  if(nonZeroBits[i]==true)
				  bitSet.set(i);
		  }
		  System.out.println("BitSet64 initialized:\n"+bitSet);
		  
		  // reading/access
		  boolean bitValue;
		  boolean[] resultingBitVector=new boolean[size];
		  for(int i=0;i<size;i++) {
			  resultingBitVector[i]=bitSet.get(i);
		  }
		  System.out.println("Resulting bit vector\n"+booleanToString(resultingBitVector));
		  
		  System.out.println("test done");
	}

	public static String booleanToString(boolean[] t) {
		String res="";
		for(int i=0;i<t.length;i++) {
			if(t[i])
				res=res+1;
			else
				res=res+0;
		}
		return res;
	}

	/**
	 * @param args
	 * vm option: -Xms256m -Xmx1024m
	 */
	public static void main(String[] args) {
		System.out.println("Comparing BitSet implementations");
		int n=100000000; // 10^6 (10^8)
		
		// timings and memory performance: tested on a Dell XT2 (Ubuntu 13.04, Java 7 64bits)
		// conclusion: BitSet64 seems slightly faster than BitSet (provided by stardard Java library)
		// boolean arrays are clearly faster (about 6-10 times), but more space consuming (1 byte/element)
		
		// benchmark for timing
		//benchJavaBitSet(n); // 0.005s, 0.002ms (0.46, 0.22)
		//benchBooleanArray(n); // 0.0006s, 0.00005ms (0.06s, 0.005s)
		//benchBitSet32(n); // 0.007s, 0.001s (0.77, 0.12)
		benchBitSet64(n); // 0.004s, 0.001s (0.34, 0.11)

		// memory storage
		Runtime runtime=Runtime.getRuntime();
		runtime.gc();
		showMemory();
		//boolean[] v=new boolean[n]; // used mem 2156k (101MB)
		//BitSet bits=new BitSet(n); // used mem 1179k (13.5MB)
		//BitSet32 bits=new BitSet32(n); // used mem 1179k (13.5MB)
		BitSet64 bits=new BitSet64(n); // used mem 1179k (13.5MB)
		showMemory();

		// check correctness of bit-vector implementations
		//correctnessBitSet32();
		//correctnessBitSet64();
	}

}
