package Jcg.test;

import Jcg.geometry.Point_3;
import Jcg.triangulations3D.Delaunay_3;
import Jcg.triangulations3D.TriangulationDSCell_3;

public class TestDelaunay_3 {
    /**
     * Testing function.
     */	
	public static void main(String[] args) {
		test6();		
	}		

		
	// Ci-dessous un exemple de bug lie a l'update de la bounding box quand on n'est pas Delaunay
	// La deuxieme instruction permet d'eviter le bug car pas de redimensionnement => pas d'inversion de simplexe
	/*private static void test0 () {
		Delaunay_3 del = new Delaunay_3();
		del.updateBB(new Point_3(1,1,1));
		Point_3[] P = new Point_3[3];
		P[0] = new Point_3(0.9388093186748128,0.9283684913785756,0.3468020578989015);
		P[1] = new Point_3(0.8660756975171705,0.05496804111962572,0.17547142955266026);
		P[2] = new Point_3(0.9516605758014842,0.693362378082091,0.2125255534606474);
		for (int i=0; i<3; i++) {
			System.out.println("inserting " + P[i]);
			TriangulationDSCell_3<Point_3> c = del.locate(P[i]);
			del.TDS.insertInCell(P[i], c);
		}
		for (int i=0; i<10000; i++) {
			Point_3 p = new Point_3(Math.random(), Math.random(), Math.random());
			System.out.println("inserting " + p);
			TriangulationDSCell_3<Point_3> c = del.locate(p);
			del.TDS.insertInCell(p, c);
		}
		for (int i=0; i<100000; i++) {
			Point_3 p = new Point_3(Math.random(), Math.random(), Math.random());
			System.out.println("locating " + p);
			del.locate(p);
		}		
		del.isValid();
		System.out.println("done.");
	}*/

	// Pour mettre au jour le probleme de la gestion de l'enveloppe convexe avec une bounding box 
	private static void test1 () {
		Delaunay_3 del = new Delaunay_3();
		Point_3[] P = new Point_3[] {
				new Point_3(0.26007989982647706,0.0770141866106312,0.6290481228869144),
				new Point_3(0.8025368614435817,0.25034263978260585,0.2243090162475877),
				new Point_3(0.9717773399026567,0.3016046408321127,0.8939414964788058),
				new Point_3(0.12952105603163888,0.07022340687643247,0.6225397225778694),
				new Point_3(0.18260316615910155,0.9272357010551369,0.9256600272248054),
				new Point_3(0.858401394305793,0.34482931382408555,0.2587439777872731),
				new Point_3(0.5463797187734596,0.7005142964718137,0.9090360998511524),
				new Point_3(0.584355844698825,0.0919257346382939,0.8265054854683757),
				new Point_3(0.9265764380442612,0.6948216656943689,0.12549598703990839),
				new Point_3(0.22959046504762892,0.552863572280065,0.2873314505740887)
		};
		for (int i=0; i<10; i++) {
			System.out.println("Inserting " + P[i]);
			del.insert(P[i]);
		}
		del.isValid();
		del.insert(new Point_3(0.8025368614435817,0.25034263978260585,0.2243090162475877));
		del.isValid();
		System.out.println("done.");
	}

	// Standard test with uniform random points in cube
	private static void test2 () {
		Delaunay_3 del = new Delaunay_3();
		for (int i=0; i<10000; i++) {
			Point_3 p = new Point_3(Math.random(), Math.random(), Math.random());
			System.out.println("inserting " + p);
			del.insert(p);
		}
		for (int i=0; i<100000; i++) {
			Point_3 p = new Point_3(Math.random(), Math.random(), Math.random());
			System.out.println("locating " + p);
			del.locate(p);
		}
		del.isValid();
		del.writeToFile("delaunay.off");
	}

	// Example of two non-coplanar lines, on which the running time is quadratic
	private static void test3 () {
		Delaunay_3 del = new Delaunay_3();
		for (int i=0; i<5000; i++) {
			Point_3 p = new Point_3(1-2*Math.random(), 1e-6-2e-6*Math.random(), -1+1e-6-2e-6*Math.random());
			Point_3 q = new Point_3(1e-6-2e-6*Math.random(), 1-2*Math.random(), 1+1e-6-2e-6*Math.random());
			System.out.println("inserting " + p);
			del.insert(p);
			System.out.println("inserting " + q);
			del.insert(q);
		}
		System.out.println("done.");
	}

	// Test for cells duals
	private static void test4 () {
		Delaunay_3 del = new Delaunay_3();
		for (int i=0; i<50; i++) {
			Point_3 p = new Point_3(Math.random(), Math.random(), Math.random());
			System.out.println("inserting " + p);
			del.insert(p);
		}
		del.isValid();

		Point_3[] points = new Point_3[2*del.cells().size()];
		int[][] edges = new int[points.length/2][2];
		int i=0;
		for (TriangulationDSCell_3<Point_3> c : del.finiteCells()) {
			points[2*i] = del.dual(c);
			points[2*i+1] = new Point_3(points[2*i].getX().doubleValue()*1.01, 
					points[2*i].getY().doubleValue()*1.01,
					points[2*i].getZ().doubleValue()*1.01);
			edges[i] = new int[] {2*i, 2*i+1};
			i++;
		}		

		del.writeToFile("delaunay3D.off");
		
		//MeshViewer m = new MeshViewer(del, true); // OLD version JAVA3D
		//m.points = points;
		//m.edges = edges;
		//m.edgeColors = new Color[edges.length];
	}

	// Test for vertices duals
	private static void test5 () {
		Delaunay_3 del = new Delaunay_3();
		for (int i=0; i<500; i++) {
			Point_3 p = new Point_3(Math.random(), Math.random(), Math.random());
			System.out.println("inserting " + p);
			del.insert(p);
		}
		del.isValid();

		del.writeToFile("delaunay3D_cube.off");
		//MeshViewer m = new MeshViewer(del, true); // OLD version JAVA3D
		//m.trianglesCollection = del.dual(del.randomFiniteVertex());
	}

	// Test for vertices duals
	private static void test6() {
		Delaunay_3 del = new Delaunay_3();
		Point_3 origin=new Point_3(0.5, 0.5, 0.5);
		for (int i=0; i<500; i++) {
			Point_3 p = new Point_3(Math.random(), Math.random(), Math.random());
			if(p.squareDistance(origin).doubleValue()<=0.3) {
				System.out.println("inserting " + p);
				del.insert(p);
			}
		}
		del.isValid();

		del.writeToFile("delaunay3D_sphere.off");
		del.writeToTetFile("delaunay3D_sphere.tet");
		//MeshViewer m = new MeshViewer(del, true); // OLD version JAVA3D
		//m.trianglesCollection = del.dual(del.randomFiniteVertex());
	}

}
