package Jcg.test;

import Jcg.geometry.*;
import Jcg.mesh.*;
import Jcg.viewer.old.Fenetre;
import Jcg.viewer.processing3d.MeshViewerProcessing;
import Jcg.io.options.*;
import Jcg.polyhedron.*;

/**
 * A class for testing the Half-edge data structure representing polyhedral surfaces.
 * 3D rendering is performed using a Mesh Viewer based on Processing (version 1.51) 
 *
 * @author Luca Castelli Aleardi (INF562, 2019)
 *
 */
public class TestPolyhedron {

	static ProcessInputParameters input;
	
	/**
	 * Testing a plane triangulation (loaded from an OFF file)
	 */    
	public static void test2D(String filename) {
    	Polyhedron_3<Point_2> planarGraph=MeshLoader.getPlanarMesh(filename);

    	//System.out.println(planarGraph.verticesToString());   	
    	//System.out.println(planarGraph.facesToString());
    	planarGraph.isValid(false); // check combinatorial validity
    	System.out.println("Test done");
    	
    	Fenetre f=new Fenetre();
    	f.addPolyhedronEdges(planarGraph);
    }
	
	/**
	 * Testing a 3D mesh (loaded from an OFF file)
	 */    
	public static void testProcessing3D(String filename) {   	
		MeshViewerProcessing.setInputFile(filename);
		MeshViewerProcessing.main(new String[] { "Jcg.viewer.MeshViewerProcessing" });    		
	}

	/**
	 * Load a mesh from an OFF file, shows statistics and check validity
	 */    
	public static void meshStatistics(String filename) {
		// build the mesh representation
		Polyhedron_3<Point_3> polyhedron3D=MeshLoader.getSurfaceMesh(filename);
		polyhedron3D.isValid(false); // check combinatorial validity
	}

	/**
	 * Load a mesh from an OFF file, shows statistics and check validity
	 */    
	public static void meshConversion(String filename) {
		// build the mesh representation
		Polyhedron_3<Point_3> polyhedron3D=MeshLoader.getSurfaceMesh(filename);
		//polyhedron3D.isValid(false); // check combinatorial validity
		IO.writePolyedronToEDGES(polyhedron3D, "output", ",");
	}

    public static void main (String[] args) {
    	// options setting
       	/*Option opP3D=new Option(0, "-Processing3D", null, "to render the mesh with Processing3D");
       	Option op2D=new Option(0, "-2D", null, "to draw the mesh in the plane");
       	String usMsg="java TestPolyhedron [-options] data.off";
    	input=new ProcessInputParameters(new Option[] {opP3D, op2D}, 1, args, usMsg);*/
    	// read the filename from input arguments
    	String filename=input.getData();
    	
    	meshStatistics(filename);
    	test2D(filename);
    	testProcessing3D(filename);
    }

}
