package Jcg.test;

import Jcg.geometry.Point_3;
import Jcg.mesh.MeshLoader;
import Jcg.polyhedron.Polyhedron_3;
import Jcg.subdivision.*;

import Jcg.viewer.processing3d.DrawPolyhedronProcessing;
import Jcg.viewer.processing3d.MeshViewerProcessing;

import processing.core.*;

/**
 * A simple PApplet for testing subdivision methods
 * 
 * @author Luca Castelli Aleardi (INF555/562, 2012-20)
 *
 */
public class TestSubdivision extends MeshViewerProcessing {
	
	/** surface mesh to be processed */
	static public Polyhedron_3<Point_3> polyhedron3D;
	
	public MeshSubdivision sm=null; // generic subdivision scheme
	public LoopSubdivision loop; // Loop subdivision scheme
	public CatmullClarkSubdivision catmullClark; // Catmull-Clark subdivision scheme
	
	/**
	 * Initialize the basic parameters of the viewer
	 */
	public void setup() {
		super.setup(); // set the arcball, 3D rendering mode and window size  
		  
		this.drawingMethod=new DrawPolyhedronProcessing(this, this.polyhedron3D); // set the 3D renderer
		this.zoomFactor=this.drawingMethod.computeScaleFactor();
		
		this.loop=new LoopSubdivision(this.drawingMethod.polyhedron3D);
		this.catmullClark=new CatmullClarkSubdivision(this.drawingMethod.polyhedron3D);
		this.sm=loop; // Loop subdivision is the scheme selected by default
	}
		 		
	/**
	 * Select the action to perform
	 */
		public void keyPressed(){
			  switch(key) {
			    case('l'):case('L'): this.sm=this.loop; this.subdivide(); break;
			    case('c'):case('C'): this.sm=this.catmullClark; this.subdivide(); break;
			    case('r'):this.renderType=(this.renderType+1)%this.renderModes; break;
			    case('+'):this.zoomIn(); break;
			    case('-'):this.zoomOut(); break;
			  }
		}

		/**
		 * Show on the screen the actions available for this program
		 */
		public void drawOptions() {
			int hF=12;
			fill(0);
			this.textMode(this.SCREEN);
			this.text("Press '+' or '-' for zoom In and Out", 10, hF);
			this.text("Press 'r' for changing the rendering mode", 10, hF*2);
			this.text("Press 'l' for performing Loop subdivision", 10, hF*3);
			this.text("Press 'c' for performing Catmull-Clark subdivision", 10, hF*4);
		}
		
		/**
		 * Perform an iteration of the subdivision
		 */
		public void subdivide() {
			this.sm.subdivide();
			this.drawingMethod.polyhedron3D.isValid(false); // check combinatorial validity
		}
		
		/**
		 * For running the PApplet as Java application
		 */
		public static void main(String args[]) {
			System.out.println("Testing subdivision methods (Loop, Catmull-Clark)");
	    	if (args.length == 0) {
	    		System.out.println("I wait for a mesh stored in OFF format");
	    		System.out.println("Usage : java TestSubdivision filename");
	    		return;
	    	}
	    	filename=args[0]; // input file (OFF format)
	    	
	    	// the input mesh must be a static field of the 3D Processing PApplet
			TestSubdivision.polyhedron3D=MeshLoader.getSurfaceMesh(filename); // load input mesh from OFF file
			
			PApplet.main(new String[] { "Jcg.test.TestSubdivision" }); // launch the 3D viewer. Warning: the input parameter must coincide with the name of the class (including package names)
		}

}
