package Jcg.util;

/**
 * @author Luca Castelli Aleardi (Ecole Polytechnique, 2013)
 * 
 * Minimal implementation of a doubly linked list, storing generic type elements
 */
public class DLinkedList<X> {
	private final DListNode<X> first, last;
	private int size;
	
	public DLinkedList() {
		this.first = new DListNode<X>(null, null, null);
		this.last = new DListNode<X>(null, null, first);
		this.first.next = last;
		size = 0;
	}
	
	public int size() { 
		return this.size; 
	}
	
	/**
	 * Check whether the list is empty
	 */
	public boolean isEmpty() {
		return this.size == 0; 
	}

	/**
	 * Return the end of the list: useful for iterating over the nodes
	 */
	public DListNode<X> getEnd() {
		return this.last;
	}

	/**
	 * Return the first node of the list
	 * The result is a null reference if the list is empty
	 */
	public DListNode<X> getFirst() {
		return this.first.getNext();
	}

	/**
	 * Return the first node of the list
	 * The result is a null reference if the list is empty
	 */
	public DListNode<X> getLast() {
		return this.last.getPrev();
	}

	/**
	 * Insert a new element at the beginning of the list
	 * @param el the element to insert
	 */
	public void addFirst(X el) {
		insertAfter(first, el);
	}
	
	/**
	 * Append a new element at the end of the list
	 * @param el the element to insert
	 */
	public void addLast(X el) {
		insertAfter(last.previous, el);
	}
	
	/**
	 * Append a new element at the beginning of the list
	 * @param el the element to insert
	 */
	public void add(X el) {
		insertAfter(last.previous, el);
	}
	
	/**
	 * Insert a new element after a given node in the list
	 * @param el the element to insert
	 * @param nPnode the place where the element will be inserted
	 */
	public void insertAfter(DListNode<X> pNode, X el) {
		DListNode<X> item = new DListNode<X>(el, pNode.next, pNode);
		item.previous.next = item;
		item.next.previous = item;
		size++;
	}

	/**
	 * Insert a new element before a given node in the list
	 * @param el the element to insert
	 * @param nPnode the place where the element will be inserted
	 */
	public void insertBefore(DListNode<X> pNode, X el) {
		DListNode<X> item = new DListNode<X>(el, pNode, pNode.previous);
		item.previous.next = item;
		item.next.previous = item;
		size++;
	}
	
	/**
	 * Remove a given node from the list
	 * @param nPnode the node to delete
	 */
	public void delete(DListNode<X> pNode) {
		if(pNode==null)
			return;
		pNode.next.previous = pNode.previous;
		pNode.previous.next = pNode.next;
		size--;
	}

	/**
	 * Return a String representing the list
	 */
	public String toString() {
		if(this.isEmpty())
			return "empty list";
		String result="[";
		DListNode<X> n=this.getFirst();
		while(n!=this.last) {
			result=result+n.getElement()+" ";
			n=n.getNext();
		}
		return result+"]";
	}

}
