package Jcg.viewer.editor2d;

import Jcg.convexhull2d.*;
import Jcg.geometry.PointCloud_2;
import Jcg.geometry.PolyLine_2;;

/**
 * Class providing the list of available algorithms.
 * The user is allowed to select a given algorithm and choose among different implementations
 * 
 * @author Luca Castelli Aleardi, Ecole Polytechnique (INF562)
 * @version dec 2012
 *
 */
public class AlgorithmList {
	// constants defining the algorithm to execute
	public static final int NO_SELECTED_ALGORITHM=-1;
	public static final int CONVEX_HULL_2=0;
	public static final int DELAUNAY_2=1;
	public static final int POINT_LOCATION_POLYHEDRON=2;
	public static final int BEZIER_2=3;
	
	public int selectedAlgorithm; // choice of the algorithm and implementation
	public Canvas2D canvas=null; // the rendering scene (containing geometric data)
	
	public ConvexHull_2 convexHull_2=null;

	/**
	 * Initialization and choice of algorithm implementations
	 */
	public AlgorithmList(Canvas2D canvas) {
		this.canvas=canvas;
		this.selectedAlgorithm=NO_SELECTED_ALGORITHM;
		this.convexHull_2=new AndrewConvexHull();
	}

	/**
	 * Select the algorithm to execute
	 * @param the integer coding the algorithm
	 */
	public void selectAlgorithm(int selection) {
		this.selectedAlgorithm=selection;
	}
	
	/**
	 * Perform the execution of the selected algorithm
	 */
	public void executeAlgorithm() {
		if(this.canvas==null)
			throw new Error("NO canvas defined");
		if(this.selectedAlgorithm==NO_SELECTED_ALGORITHM) {
			return;
		}
		if(this.selectedAlgorithm==CONVEX_HULL_2) {
			if(this.canvas.graph==null) {
				System.out.println("No graph nor point cloud defined");
				return;
			}
			// retrieve input points
			PointCloud_2 inputPoints=new PointCloud_2(this.canvas.graph.listOfPoints());
			PointCloud_2 extremalPoints=this.convexHull_2.computeConvexHull(inputPoints);
			PolyLine_2 edges=new PolyLine_2(extremalPoints);
			System.out.println("convex hull computed");
			
			// update the rendered scene
			this.canvas.initRenderedObjects();
			this.canvas.addPoints(extremalPoints.listOfPoints());
			this.canvas.addSegments(edges.computeSegments());
			this.canvas.repaint();
		}
	}
	
}
