package Jcg.viewer.editor2d;

import java.awt.*;
import java.util.Collection;

import Jcg.geometry.Point_2;
import Jcg.geometry.Segment_2;
import Jcg.geometry.Vector_2;
import Jcg.graph.*;
import Jcg.polyhedron.*;
import Jcg.viewer.Draw2D;


/**
 * A class for drawing points, segments, graphs and planar meshes, in a cartesian frame
 * 
 * @author Luca Castelli Aleardi, Ecole Polytechnique (INF562)
 * @version dec 2012
 */
public class Draw extends Draw2D {

	Graphics g;
	Point_2 a,b; // range of the window (left bottom and right top corners)
	int width, height; // size of the window (number of pixels)
	
	boolean[] translate_x, translate_y;
	
	public Draw(Graphics g,Point_2 a,Point_2 b,int w, int h) {
		if(g==null)
			throw new Error("Canvas2D error: Graphics not defined");
		if(a==null || b==null)
			throw new Error("Canvas2D error: Window ranges not defined");
		if(w<=0 || h<=0)
			throw new Error("Canvas2D error: Window size not valid");
		if(a.getX().doubleValue()>=b.getX().doubleValue())
			throw new Error("Canvas2D error: x range not valid");
		if(a.getY().doubleValue()>=b.getY().doubleValue())
			throw new Error("Canvas2D error: y range not valid");
		this.g=g; 
		this.a=a; 
		this.b=b; 
		this.width=w; 
		this.height=h;
	}
	
	public Draw() {}
	
	
	public void resize(Point_2 leftBottom, Point_2 rightTop) {
		this.a=leftBottom; 
		this.b=rightTop; 		
	}
	
	public void drawSegment(Point_2 p, Point_2 q) {
		Point pI=this.getPoint(p);
		Point qI=this.getPoint(q);
		
		g.drawLine(pI.x, this.height-pI.y, qI.x, this.height-qI.y);
	}
	
	/**
	 * Draw a point in the window (as a small circle)
	 */
	public void drawPoint(Point_2 v) {
		double x=v.getX().doubleValue();
		double y=v.getY().doubleValue();

		int i=(int) (this.width*( (x-a.getX().doubleValue()) / (b.getX().doubleValue()-a.getX().doubleValue()) ));
		int j=(int) (this.height*( (y-a.getY().doubleValue()) / (b.getY().doubleValue()-a.getY().doubleValue()) ));
		
		g.fillOval(i-1,this.height-j-1,4,4);
	}
	
	/**
	 * Draw a point in the window (as a small circle)
	 */
	public void drawTinyPoint(Point_2 v) {
		double x=v.getX().doubleValue();
		double y=v.getY().doubleValue();

		int i=(int) (this.width*( (x-a.getX().doubleValue()) / (b.getX().doubleValue()-a.getX().doubleValue()) ));
		int j=(int) (this.height*( (y-a.getY().doubleValue()) / (b.getY().doubleValue()-a.getY().doubleValue()) ));
		
		g.fillOval(i-1,this.height-j-1,1,1);
	}
	
	/**
	 * Draw points corresponding to an integer grid
	 */
	public void drawGrid() {
		int xmin, xmax, ymin, ymax;
		xmin=(int)this.a.getX().doubleValue();
		xmax=(int)this.b.getX().doubleValue();
		ymin=(int)this.a.getY().doubleValue();
		ymax=(int)this.b.getY().doubleValue();
		Color currentColor=this.g.getColor();
		this.g.setColor(Color.lightGray);
		for(int i=xmin;i<xmax;i++) {
			for(int j=ymin;j<ymax;j++) {
				Point_2 gridPoint=new Point_2(i, j);
				this.drawTinyPoint(gridPoint);
			}
		}
		g.setColor(currentColor);
	}

	
	/**
	 * Return a point corresponding to a given position in the window
	 * The input are the integer coordinates of the pixel
	 */
	public Point_2 getPoint(int i, int j) {
		double x, y;
		j=this.height-j; // y coordinates are increasing from top to bottom

		double xRange=b.getX().doubleValue()-a.getX().doubleValue(); // width and height of the window
		double yRange=b.getY().doubleValue()-a.getY().doubleValue();

		x=a.getX().doubleValue()+i*(xRange/this.width);
		y=a.getY().doubleValue()+j*(yRange/this.height);
		return new Point_2(x, y);
	}

	/**
	 * Return the integer coordinates of a pixel corresponding to a given point
	 */
	public Point getPoint(Point_2 v) {
		double x=v.getX().doubleValue(); // coordinates of point v
		double y=v.getY().doubleValue();
		double xRange=b.getX().doubleValue()-a.getX().doubleValue(); // width and height of the window
		double yRange=b.getY().doubleValue()-a.getY().doubleValue();
		int i= (int) (this.width*( (x-a.getX().doubleValue()) / xRange ));
		int j= (int) (this.height*( (y-a.getY().doubleValue()) / yRange ));
		
		return new Point(i,j);
	}

	public void draw(int type) {
		throw new Error("To be completed");
	}
	
	
	
	//----------------------------------
	//----------------------------------
	//----------------------------------

	
	
    public void fillTriangle(int[] x, int[] y, Color colorTriangle, Color e1, Color e2, Color e3,boolean filled) {
    	g.setColor(colorTriangle);
    	if(filled==true && colorTriangle!=Color.white) g.fillPolygon(x,y,3);
    	
		g.setColor(e1); g.drawLine(x[0],y[0],x[1],y[1]);
		g.setColor(e3); g.drawLine(x[0],y[0],x[2],y[2]);
		g.setColor(e2); g.drawLine(x[1],y[1],x[2],y[2]);
    }

    public void fillTriangle(Point_2 p1, Point_2 p2, Point_2 p3, Color triangle) {
    	//throw new Error("To be completed");  	
    	g.setColor(triangle);

		int[] x=new int[3];
		int[] y=new int[3];
		double denominatorX=b.getX().doubleValue()-a.getX().doubleValue();
		double denominatorY=b.getY().doubleValue()-a.getY().doubleValue();
		
		x[0]= (int) (this.width*( (p1.getX().doubleValue()-a.getX().doubleValue()) / denominatorX ));
		y[0]= this.height - (int) (this.height*( (p1.getY().doubleValue()-a.getY().doubleValue()) / denominatorY ));
		
		x[1]= (int) (this.width*( (p2.getX().doubleValue()-a.getX().doubleValue()) / denominatorX ));
		y[1]= this.height - (int) (this.height*( (p2.getY().doubleValue()-a.getY().doubleValue()) / denominatorY ));

		x[2]= (int) (this.width*( (p3.getX().doubleValue()-a.getX().doubleValue()) / denominatorX ));
		y[2]= this.height - (int) (this.height*( (p3.getY().doubleValue()-a.getY().doubleValue()) / denominatorY ));
    	
    	g.fillPolygon(x,y,3);
    }

    // draw an arrow on the vector v-u (oriented toward v)
    public void drawArrow(Point_2 u, Point_2 v, Color arrowColor) {
    	Point_2 p0, p1, p2, c;
    	
    	double b=0.6;
    	double x_coord=(1-b)*u.getX().doubleValue()+b*v.getX().doubleValue();
    	double y_coord=(1-b)*u.getY().doubleValue()+b*v.getY().doubleValue();
    	c=new Point_2(x_coord, y_coord);

    	double radius=Math.sqrt(u.squareDistance(v).doubleValue());
    	double dxNormalized=(v.getX().doubleValue()-u.getX().doubleValue())/radius;
    	double dyNormalized=(v.getY().doubleValue()-u.getY().doubleValue())/radius;
    	double alpha=Math.acos(dxNormalized);
    	if(dyNormalized<0.) alpha=-1.*alpha;
    	
    	double factor=0.01;
    	p0=new Point_2(c);
    	p0.translateOf(new Vector_2(factor*Math.cos(alpha-Math.PI/2.), factor*Math.sin(alpha-Math.PI/2.)));
    	p1=new Point_2(c);
    	p1.translateOf(new Vector_2(factor*Math.cos(alpha+Math.PI/2.), factor*Math.sin(alpha+Math.PI/2.)));
    	p2=new Point_2(c);
    	p2.translateOf(new Vector_2(3*factor*Math.cos(alpha), 3*factor*Math.sin(alpha)));
    	
    	if(arrowColor!=null)
    		g.setColor(arrowColor);
    	this.fillTriangle(p0, p1, p2, arrowColor);
    	g.setColor(Color.black);
    }

    public void fillTriangle(int[] x, int[] y, Color triangle, Color e1, Color e2, Color e3, double epsilon, boolean filled) {
    	g.setColor(triangle);
    	if(filled==true) g.fillPolygon(x,y,3);
    	
    	int[] v0,v1,v2;
    	v0=this.getLinearCombination(x,y,1.-epsilon,epsilon/2.,epsilon/2.);
    	v1=this.getLinearCombination(x,y,epsilon/2.,1.-epsilon,epsilon/2.);
    	v2=this.getLinearCombination(x,y,epsilon/2.,epsilon/2.,1.-epsilon);
    	
    	int[] edge_x=new int[4], edge_y=new int[4];
		
		edge_x[0]=x[0]; edge_x[1]=x[1]; edge_x[2]=v1[0]; edge_x[3]=v0[0];
		edge_y[0]=y[0]; edge_y[1]=y[1]; edge_y[2]=v1[1]; edge_y[3]=v0[1];
		g.setColor(e1); 
		g.fillPolygon(edge_x,edge_y,4);	
		//
		g.setColor(Color.black); g.drawLine(x[0],y[0],x[1],y[1]);
		

		edge_x[0]=x[0]; edge_x[1]=x[2]; edge_x[2]=v2[0]; edge_x[3]=v0[0];
		edge_y[0]=y[0]; edge_y[1]=y[2]; edge_y[2]=v2[1]; edge_y[3]=v0[1];
		g.setColor(e3); 
		g.fillPolygon(edge_x,edge_y,4);	
		//
		g.setColor(Color.black); g.drawLine(x[0],y[0],x[2],y[2]);

		edge_x[0]=x[1]; edge_x[1]=x[2]; edge_x[2]=v2[0]; edge_x[3]=v1[0];
		edge_y[0]=y[1]; edge_y[1]=y[2]; edge_y[2]=v2[1]; edge_y[3]=v1[1];		
		g.setColor(e2); 
		g.fillPolygon(edge_x,edge_y,4);	
		//
		g.setColor(Color.black); g.drawLine(x[1],y[1],x[2],y[2]);
    }
    
    public int[] getLinearCombination(int[] x, int[] y, double alpha, double beta, double gamma) {
    	// the sum of alpha, beta and gamma must be 1
    	int[] v=new int[2]; // v is a linear combination of the 3 input points
    	
    	v[0]=(int)(alpha*x[0]+beta*x[1]+gamma*x[2]);
    	v[1]=(int)(alpha*y[0]+beta*y[1]+gamma*y[2]);
    	
    	return v;
    }
	
	public void drawSegment(Point_2 p, Point_2 q, Color c) {
		if(p==null || q==null)
			return;
		Point pI=this.getPoint(p);
		Point qI=this.getPoint(q);
		
		if(c!=null)
			g.setColor(c);
		else
			g.setColor(Color.black);
		g.drawLine(pI.x, this.height-pI.y, qI.x, this.height-qI.y);
		g.setColor(Color.black);
	}
	
	public void drawSegment(Segment_2 s, Color c) {		
		if(c!=null)
			g.setColor(c);
		else
			g.setColor(Color.black);
		this.drawSegment((Point_2)s.source(), (Point_2)s.target(), c);
		g.setColor(Color.black);
	}
	
	/**
	 * Draw a circle, with a given color and size
	 */
	public void drawCircle(Point_2 v, int r, Color c) {
		double x=v.getX().doubleValue();
		double y=v.getY().doubleValue();

		int i=(int) (this.width*( (x-a.getX().doubleValue()) / (b.getX().doubleValue()-a.getX().doubleValue()) ));
		int j=(int) (this.height*( (y-a.getY().doubleValue()) / (b.getY().doubleValue()-a.getY().doubleValue()) ));
		
		Color currentColor=this.g.getColor();
		g.setColor(c);
		g.fillOval(i-2,this.height-j-2,r,r);
		g.setColor(currentColor);
	}
	
	public void drawLabel(Point_2 v, String label) {
		double x=v.getX().doubleValue();
		double y=v.getY().doubleValue();
		
		int i= (int) (this.width*( (x-a.getX().doubleValue()) / (b.getX().doubleValue()-a.getX().doubleValue()) ));
		int j= (int) (this.height*( (y-a.getY().doubleValue()) / (b.getY().doubleValue()-a.getY().doubleValue()) ));
		
		g.setColor(Color.blue);
		//g.fillOval(n1,h-m1,9,9);
		
		Font font=Font.getFont("SANS_SERIF");
		g.setFont(font);
		g.drawString(label,i+3,this.height-j+7);
		g.setColor(Color.black);
	}

	
	public String toString() {
		String result="Canvas 2D - ";
		result=result+"window range: "+a+" - "+b;
		result=result+", width,height: ["+width+","+height+"]";
		
		return result;
	}
	
	/**
	 * Draw the vertices and edges of a graph
	 * 
	 * @param graph The input graph (GeometricGraph_2)
	 */
	public void drawGraph(GeometricGraph_2 graph) {
		// drawing edges
		Collection<Point_2[]> edges=graph.computeEdges();
		for(Point_2[] segment: edges) {
			this.drawSegment(segment[0], segment[1]);
		}
		// drawing vertices
		for(Node<Point_2> v: graph.vertices) {
			Point_2 p=graph.getPoint(v);
			this.drawPoint(p);
		}
	}
	
	/**
	 * Draw the edges of a planar mesh (halfedge data structure)
	 * 
	 * @param graph The input graph (GeometricGraph_2)
	 */
	public void drawMesh(Polyhedron_3<Point_2> mesh) {
		// drawing edges
		for(Halfedge<Point_2> e: mesh.halfedges) {
			this.drawSegment(e.getVertex().getPoint(), e.getOpposite().getVertex().getPoint());
		}
		// drawing vertices
		/*for(Node<Point_2> v: graph.vertices) {
			Point_2 p=graph.getPoint(v);
			this.drawPoint(p);
		}*/
	}

	/**
	 * Draw the colored edges of a planar mesh (halfedge data structure)
	 * 
	 * @param graph The input graph (GeometricGraph_2)
	 */
	public void drawMesh(Polyhedron_3<Point_2> mesh, Color[] colors) {
		// drawing edges
		if(colors==null)
			for(Halfedge<Point_2> e: mesh.halfedges)
				this.drawSegment(e.getVertex().getPoint(), e.getOpposite().getVertex().getPoint());
		else {
			int i=0;
			for(Halfedge<Point_2> e: mesh.halfedges) {
				this.drawSegment(e.getVertex().getPoint(), e.getOpposite().getVertex().getPoint(), colors[i]);
				i++;
			}
		}
	}


}
