package Jcg.viewer.editor2d;

import java.awt.*;
import java.applet.Applet;
import java.awt.event.*;
import java.awt.FileDialog;

import Jcg.polyhedron.*;
import Jcg.geometry.*;
import Jcg.graph.GeometricGraph_2;

/**
 * @author Luca Castelli Aleardi, Ecole Polytechnique (INF562)
 * @version dec 2012
 * Simple editor for visualizing and editing planar graphs and relatives
 */
public class Editor2D extends Frame implements ActionListener {
	
	// GUI Widgets
	Button addVertex;
	Button removeVertex;
	Button moveVertex;
	Button addEdge;
	
	public Canvas2D canvas; // canvas where objects are drawn
	public ControlBar controlBar; // control bar providing buttons (zoom in, zoom out, draw bounding box, ...)
	
	public Editor2D() { 
		  super("Editor 2D - Jcg library (Ecole Polytechnique)"); 
		  
		  //  creating and setting menus
		  FileMenu fileMenu = new FileMenu(this);
		  AlgorithmsMenu algorithmsMenu=new AlgorithmsMenu(this);
		  HelpMenu helpMenu = new HelpMenu(this); 
		  MenuBar menubar = new MenuBar(); 
		  menubar.add(fileMenu);
		  menubar.add(algorithmsMenu);
		  menubar.add(helpMenu); 
		  setMenuBar(menubar); 
		  
		  init();
		  
		  addWindowListener(new WindowAdapter() { 
			  public void windowClosing(WindowEvent e) { 
				  exit(); 
			  } 
		  }); 
	} 
	
	public void init() {
		setLayout(new BorderLayout());
		
		Panel controlPanel = new Panel();
		controlPanel.setBackground(Color.gray);
		add("North", controlPanel);

		// add buttons
		addVertex = new Button("add vertex");
		controlPanel.add(addVertex);
		addVertex.addActionListener(this);

		removeVertex = new Button("remove vertex");
		controlPanel.add(removeVertex);
		removeVertex.addActionListener(this);

		moveVertex = new Button("move vertex");
		controlPanel.add(moveVertex);
		moveVertex.addActionListener(this);

		addEdge = new Button("connect vertices");
		controlPanel.add(addEdge);
		addEdge.addActionListener(this);

		// add the canvas
		Panel centerPanel = new Panel();
		centerPanel.setLayout(new BorderLayout());
		
		this.canvas = new Canvas2D();
		this.controlBar=new ControlBar(this.canvas);
		centerPanel.add(this.canvas);
		centerPanel.add("North", this.controlBar);
		
		add("Center", centerPanel);
	}
	
	public void actionPerformed(ActionEvent e) {
	
		if (e.getSource() == addVertex) {
			this.canvas.actionToPerform=Canvas2D.ADD_VERTEX;
			repaint();		// repaint this applet
			canvas.repaint();	// also repaint the canvas
		}
		if (e.getSource() == removeVertex) {
			this.canvas.actionToPerform=Canvas2D.REMOVE_VERTEX;
			repaint();		// repaint this applet
			canvas.repaint();	// also repaint the canvas
		}
		if (e.getSource() == moveVertex) {
			this.canvas.actionToPerform=Canvas2D.MOVE_VERTEX;
			repaint();		// repaint this applet
			canvas.repaint();	// also repaint the canvas
		}
		if (e.getSource() == addEdge) {
			this.canvas.actionToPerform=Canvas2D.ADD_EDGE;
			repaint();		// repaint this applet
			canvas.repaint();	// also repaint the canvas
		}
}
	
	
	  public void exit() { 
		  setVisible(false); 
		  dispose(); 
		  System.exit(0); 
	  } 

	  public static void main(String args[]) { 
		  Editor2D f = new Editor2D(); 
		  f.setVisible(true); 
		  f.setSize(500, 500);
		  f.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
				    System.exit(0);
				}
		  });
		  
		  f.show();
	  } 

}