package Jcg.viewer.processing2d;


import Jcg.geometry.Point_2;

import java.util.LinkedList;

import processing.core.*;

/**
 * 
 * A class for drawing 2D objects (points, segments, ...) in a 2D Processing frame
 * 
 * @author Luca Castelli Aleardi
 *
 */
public class Canvas2DProcessing extends PApplet {
	
	Draw2DProcessing drawingMethod2D; // drawing method
	
	LinkedList<Point_2> points=new LinkedList<Point_2>(); // the input points to interpolate
	final static double epsilon=4.;
	
	public Point_2 selectedPoint=null; // point selected with mouse events

	  public void setup() {
		  System.out.println("Canvas 2D Processing");
		  size(400,400); // size of the window
		  
		  // choose the scheme and curves to draw
	      this.drawingMethod2D=new Draw2DProcessing(this);
	  }

	  public void draw() {
	    background(220);
	    
	    if(points.isEmpty()) return; // no points to interpolate
	    for(Point_2 p: this.points)
	    	this.drawingMethod2D.drawPoint(p);
	    
	    this.drawingMethod2D.drawSegment(new Point_2(0., 0.), new Point_2(1., 0.));
	  }

	  public void removePoint(int x, int y) {
		  int index=findPoint(x, y);
		  if(index>=0 && index<this.points.size())
			  this.points.remove(index);
	  }
	  
	  public int findPoint(int x, int y) {
		  Point_2 p=new Point_2(x, y);
		  
		  int index=0;
		  boolean found=false;
		  for(Point_2 q:this.points) {
			  if(q.squareDistance(p).doubleValue()<epsilon) {
				  found=true;
				  break;
			  }
			  index++;
		  }
		  if(found==true)
			  return index;
		  else
			  return -1;
	  }

	  public Point_2 selectPoint(int x, int y) {
		  Point_2 p=new Point_2(x, y);
		  
		  int index=0;
		  boolean found=false;
		  for(Point_2 q:this.points) {
			  if(q.squareDistance(p).doubleValue()<epsilon) {
				  found=true;
				  break;
			  }
			  index++;
		  }
		  if(found==true)
			  return this.points.get(index);
		  else
			  return null;
	  }

	  public void mouseClicked() {
		  Point_2 p=new Point_2(mouseX, mouseY);
		  
		  if(mouseButton==LEFT && this.selectedPoint==null)
			  this.points.add(p);
		  else if(mouseButton==RIGHT)
			  removePoint(mouseX, mouseY);
		  
		  //this.drawingMethod2D.updateInputPoints(this.points);
	  }

	  public void mousePressed() {
		  this.selectedPoint=selectPoint(mouseX, mouseY);
	  }
	  
	  public void mouseReleased() {
		  
		  if(this.selectedPoint!=null) {
			  this.selectedPoint.setX((double)mouseX);
			  this.selectedPoint.setY((double)mouseY);
		  }
	  }
	  
	  public void mouseDragged() {
	  }
	  
		/**
		 * For running the PApplet as Java application
		 */
		public static void main(String args[]) {
			System.out.println("Canvas2DProcessing");
			PApplet.main(new String[] { "Jcg.viewer.processing2d.Canvas2DProcessing" });
			//PApplet.main(new String[] { "DrawCurve" });
		}

}
